@@ -1,6 +1,10 @@
-import os
+# -*- coding: utf-8
+from __future__ import unicode_literals
 
-__version__ = (1,1,0)
+import os
+import codecs
+
+__version__ = (1,1,1)
 
 class FragmentsError(Exception): pass
 
@@ -14,3 +18,6 @@ def _iterate_over_files(args, config):
     else:
         for f in sorted(config['files']):
             yield os.path.join(config.root, f)
+
+def _smart_open(path, mode='r'):
+    return codecs.open(path, mode=mode, encoding='utf8')
