@@ -1,10 +1,13 @@
+# -*- coding: utf-8
+from __future__ import unicode_literals
+
 import os
 import sys
-import uuid
+import hashlib
 import argparse
 #import difflib
 
-from . import __version__, FragmentsError, _iterate_over_files
+from . import __version__, FragmentsError, _iterate_over_files, _smart_open
 from .config import FragmentsConfig, configuration_directory_name, find_configuration, ConfigurationFileCorrupt, ConfigurationFileNotFound, ConfigurationDirectoryNotFound
 from .diff import _full_diff
 from .apply import apply
@@ -27,8 +30,9 @@ def help(*args):
 
 
 def init(*args):
-    """Initialize a new fragments repository, in a directory named _fragments/."""
+    """Initialize a new fragments repository. Repository is in a directory named _fragments/, created in either the current working directory, or FRAGMENTS_ROOT if specified."""
     parser = argparse.ArgumentParser(prog="%s %s" % (__package__, init.__name__), description=init.__doc__)
+    parser.add_argument('FRAGMENTS_ROOT', help="root directory in which to create the _fragments/ directory", nargs="?")
     args = parser.parse_args(args)
 
     try:
@@ -41,7 +45,10 @@ def init(*args):
         config = FragmentsConfig(autoload=False)
         config.dump()
     except ConfigurationDirectoryNotFound as exc:
-        configuration_parent = os.path.split(os.getcwd())[0]
+        if args.FRAGMENTS_ROOT:
+            configuration_parent = os.path.realpath(args.FRAGMENTS_ROOT)
+        else:
+            configuration_parent = os.path.split(os.getcwd())[0]
         if os.access(configuration_parent, os.R_OK|os.W_OK):
             configuration_path = os.path.join(configuration_parent, configuration_directory_name)
             os.mkdir(configuration_path)
@@ -115,7 +122,6 @@ def follow(*args):
     args = parser.parse_args(args)
 
     config = FragmentsConfig()
-    random_uuid = uuid.uuid4()
     for filename in set(args.FILENAME):
         fullpath = os.path.realpath(filename)
         if fullpath.startswith(config.root):
@@ -124,9 +130,9 @@ def follow(*args):
                 yield "'%s' is already being followed" % os.path.relpath(filename)
                 continue
             if os.access(fullpath, os.W_OK|os.R_OK):
-                file_uuid = uuid.uuid5(random_uuid, key)
-                config['files'][key] = file_uuid
-                yield "'%s' is now being followed, UUID %s" % (os.path.relpath(filename), file_uuid)
+                file_sha = hashlib.sha256(('%s:%s' % (__package__, key)).encode('utf8')).hexdigest()
+                config['files'][key] = file_sha
+                yield "'%s' is now being followed (SHA-256: '%s')" % (os.path.relpath(filename), file_sha)
             else:
                 yield "Could not access '%s' to follow it" % os.path.relpath(filename)
         else:
@@ -146,10 +152,10 @@ def forget(*args):
         if fullpath.startswith(config.root):
             key = os.path.relpath(fullpath, config.root)
             if key in config['files']:
-                file_uuid = config['files'][key]
-                uuid_path = os.path.join(config.directory, file_uuid)
-                if os.access(os.path.join(config.directory, file_uuid), os.W_OK|os.R_OK):
-                    os.unlink(uuid_path)
+                file_sha = config['files'][key]
+                sha_path = os.path.join(config.directory, file_sha)
+                if os.access(os.path.join(config.directory, file_sha), os.W_OK|os.R_OK):
+                    os.unlink(sha_path)
                     yield "'%s' is no longer being followed" % os.path.relpath(filename)
                 else:
                     yield "'%s' was never committed and will not be followed" % os.path.relpath(filename)
@@ -211,9 +217,9 @@ def diff(*args):
             repo_lines = []
             curr_lines = []
             if s in 'MA':
-                curr_lines = open(curr_path, 'r').readlines()
+                curr_lines = _smart_open(curr_path, 'r').readlines()
             if s in 'MD':
-                repo_lines = open(os.path.join(config.directory, config['files'][key]), 'r').readlines()
+                repo_lines = _smart_open(os.path.join(config.directory, config['files'][key]), 'r').readlines()
             weave = Weave()
             weave.add_revision(1, repo_lines, [])
             weave.add_revision(2, curr_lines, [])
@@ -240,8 +246,8 @@ def commit(*args):
         s = _file_status(config, curr_path)
         if s in 'MA':
             repo_path = os.path.join(config.directory, config['files'][key])
-            with open(repo_path, 'w') as repo_file:
-                repo_file.write(open(curr_path, 'r').read())
+            with _smart_open(repo_path, 'w') as repo_file:
+                repo_file.write(_smart_open(curr_path, 'r').read())
             os.utime(repo_path, os.stat(curr_path)[7:9])
             yield "'%s' committed" % os.path.relpath(curr_path)
         elif s == 'D':
@@ -267,8 +273,8 @@ def revert(*args):
         s = _file_status(config, curr_path)
         if s in 'MD':
             repo_path = os.path.join(config.directory,  config['files'][key])
-            with open(curr_path, 'w') as curr_file:
-                curr_file.write(open(repo_path, 'r').read())
+            with _smart_open(curr_path, 'w') as curr_file:
+                curr_file.write(_smart_open(repo_path, 'r').read())
             os.utime(curr_path, os.stat(repo_path)[7:9])
             yield "'%s' reverted" % key
         elif s == 'A':
@@ -316,12 +322,12 @@ def fork(*args):
 
     weave = Weave()
 
-    new_lines = open(old_filenames[0], 'r').readlines()
+    new_lines = _smart_open(old_filenames[0], 'r').readlines()
     previous_revision = 1
     weave.add_revision(previous_revision, new_lines, [])
     for old_name in old_filenames[1:]:
         current_revision = previous_revision + 1
-        weave.add_revision(current_revision, open(old_name, 'r').readlines(), [])
+        weave.add_revision(current_revision, _smart_open(old_name, 'r').readlines(), [])
         new_lines = []
         diff_output = weave.merge(previous_revision, current_revision)
         i = 0
@@ -341,12 +347,17 @@ def fork(*args):
         previous_revision = current_revision + 1
         weave.add_revision(previous_revision, new_lines, [])
 
-    with open(new_path, 'w') as new_file:
+    with _smart_open(new_path, 'w') as new_file:
         new_file.writelines(new_lines)
     yield "Forked new file in '%s', remember to follow and commit it" % os.path.relpath(args.TARGET_FILENAME)
     config.dump()
 
 
+try:
+    raw_input
+except NameError: # pragma: no cover # Python 3 support
+    raw_input = input
+
 def _main(): # pragma: no cover
     from . import commands
     print("%s version %s.%s.%s" % ((__package__,) + __version__))
@@ -374,11 +385,11 @@ def _main(): # pragma: no cover
                     if isinstance(l, color.Prompt):
                         response = raw_input(l)
                         l = command_generator.send(response.strip())
-                    print(l)
+                    print(str(l))
                 except StopIteration:
                     break
-        except ExecutionError as exc:
-            sys.exit(exc.message)
+        except FragmentsError as exc:
+            sys.exit(exc.args[0])
         except KeyboardInterrupt:
             pass
 
