@@ -1,7 +1,9 @@
+# -*- coding: utf-8
+from __future__ import unicode_literals
+
 import os
 import sys
 import json
-import uuid
 
 from . import FragmentsError, __version__
 
@@ -16,13 +18,6 @@ class ConfigurationFileNotFound(ConfigurationError): pass
 class ConfigurationFileCorrupt(ConfigurationError): pass
 
 
-class UUIDEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, uuid.UUID):
-            return str(obj)
-        return json.JSONEncoder.default(self, obj)
-
-
 def find_configuration(current=None):
     current = current or os.getcwd()
     path = current
@@ -66,4 +61,4 @@ class FragmentsConfig(dict):
 
     def dump(self):
         self['version'] = __version__
-        open(self.path, 'w').write(json.dumps(self, sort_keys=True, indent=4, cls=UUIDEncoder))
+        open(self.path, 'w').write(json.dumps(self, sort_keys=True, indent=4))
